package ch.zhaw.projekt2.turnierverwaltung;

import ch.zhaw.projekt2.turnierverwaltung.main.gameScheduleView.GameController;
import ch.zhaw.projekt2.turnierverwaltung.main.gameScheduleView.GameDecorator;
import javafx.fxml.FXMLLoader;
import javafx.geometry.Insets;
import javafx.scene.control.Label;
import javafx.scene.layout.*;
import javafx.scene.paint.Color;
import javafx.scene.text.Font;
import java.io.IOException;
import java.util.logging.Logger;

/**
 * Class to load the views from FXML Files and switch between the views.
 */
public class Factory {
    private TournamentDecorator tournamentDecorator;
    private FileIO fileIO;
    private static LanguageConfigurator languageConfigurator;

    private static final Logger logger = Logger.getLogger(Factory.class.getCanonicalName());

    /**
     * Contructor to create Factory instance.
     * @param fileIO the fileIO instance which saves and reads the tournament from files.
     * @param tournamentDecorator the touramanetDecorator class to access the tournament.
     */
    public Factory(FileIO fileIO, TournamentDecorator tournamentDecorator, LanguageConfigurator languageConfigurator) {
        this.fileIO = fileIO;
        this.tournamentDecorator = tournamentDecorator;
        this.languageConfigurator = languageConfigurator;
    }

    /**
     * Setter Method of tournament
     * @param tournament the new tournament Object.
     */
    public void setTournament(Tournament tournament) {
        this.tournamentDecorator.setTournament(tournament);
    }

    /**
     * Method to load the main Window (without the content in the center)
     * @return the boarder Pane which is loaded.
     */
    public BorderPane loadMainWindow() {
        FXMLLoader loader = new FXMLLoader(getClass().getResource("MainWindow.fxml"));
        try {
            BorderPane pane = loader.load();
            FXController controller = loader.getController();
            controller.setup(languageConfigurator);
            return pane;
        } catch (IOException e) {
            logger.warning("Fatal error program can not continue after this: " + e );
            e.printStackTrace();
        }
        return null;
    }

    /**
     * Class which loads all views of Enum View
     * @param factoryDecorator the FactoryDecorator which is used to setup the controller.
     * @param pane the pane where the loaded view will be visible
     */
    public void loadAllViews(FactoryDecorator factoryDecorator, BorderPane pane){
        for(View view : View.values()){
            try {
                view.loadView(tournamentDecorator, fileIO, factoryDecorator, pane);
            } catch (IOException e) {
                e.printStackTrace();
                //TODO Handle and logging.
            }
        }
    }

    /**
     * Method to show the view TournamentList
     * @param pane the Pane where the View will be visible
     */
    public void showTournamentList(BorderPane pane) {
        tournamentDecorator.setTournament(null);
        setCenterOfBorderPane(pane, View.tournamentList);
    }

    /**
     * Method to show the view ParticipantFormular
     * @param pane the Pane where the View will be visible
     */
    public void showParticipantFormular(BorderPane pane) {
        setCenterOfBorderPane(pane, View.participantFormular);
    }

    /**
     * Method to show the view PlacesFormular
     * @param pane the Pane where the View will be visible
     */
    public void showPlacesFormular(BorderPane pane) {
        setCenterOfBorderPane(pane, View.placesFormular);
    }

    /**
     * Method to show the view GameScheduler
     * @param pane the Pane where the View will be visible
     */
    public void showGameScheduler(BorderPane pane) {
        setCenterOfBorderPane(pane, View.gameScheduler);
    }

    /**
     * Method to change the view in the center of the boarder pane in the mainWindow
     * @param pane the Pane where the View will be visible
     * @param view the view which should be visible
     */
    private void setCenterOfBorderPane(BorderPane pane, View view) {
        FXController controller = null;
        pane.setCenter(view.getPane());
        resetFooter(pane, true);
    }

    /**
     * Method to load a Game View
     * @param box the box where the game view should be added.
     * @param gameDecorator the gameDecorator instance of the game.
     * @param factoryDecorator the factoryDecorator instance.
     * @return the controller of the GameView
     */
    public GameController loadGameView(VBox box, GameDecorator gameDecorator, FactoryDecorator factoryDecorator) {
        try {
            FXMLLoader loader = new FXMLLoader(getClass().getResource("gameScheduleView/Game.fxml"));
            box.getChildren().add(loader.load());
            GameController controller = loader.getController();
            controller.setup(tournamentDecorator, fileIO, factoryDecorator, box, gameDecorator, languageConfigurator);
            return controller;
        } catch (IOException e) {
            logger.warning("Fatal error program can not continue after this: " + e );
            e.printStackTrace();
        }
        return null;
    }

    /**
     * Method to print information for the user to the footer of mainwindow.
     * @param pane the pane where the footer should be shown.
     * @param msg the text to show.
     * @param error true if it's a error message.
     */
    public void printMessageToFooter(BorderPane pane, String msg, boolean error) {
        VBox bottom = (VBox) pane.getBottom();
        Label label = new Label();
        VBox innerVbox;
        label.setText(msg);

        if (error) {
            innerVbox = (VBox) bottom.getChildren().get(0);
            resetFooter(pane, true);
            label.setTextFill(Color.RED);
            label.setFont(new Font(20));
            innerVbox.setBorder(new Border(new BorderStroke(Color.valueOf("#FF0000"), BorderStrokeStyle.SOLID, CornerRadii.EMPTY, BorderWidths.DEFAULT)));
            innerVbox.getChildren().add(label);

        } else {
            innerVbox = (VBox) bottom.getChildren().get(1);
            resetFooter(pane,false);
            label.setTextFill(Color.GREEN);
            label.setFont(new Font(10));
            label.setPadding(new Insets(0,10,0,0));
            innerVbox.getChildren().add(label);

        }
    }

    /**
     * Method to remove the messages in the footer.
     * @param pane the pane were the footer should be reseted
     * @param error true if the error message should be cleared.
     */
    public void resetFooter(BorderPane pane,boolean error) {
        VBox bottom = (VBox) pane.getBottom();
        VBox vBox;
        if (error) {
            vBox = (VBox) bottom.getChildren().get(0);
        } else {
            vBox = (VBox) bottom.getChildren().get(1);
        }
        vBox.getChildren().clear();
        vBox.setBorder(null);
    }


    public LanguageConfigurator getLanguageConfigurator() {
        return languageConfigurator;
    }

      /**
     * Enum of all views which can be set to the center of the mainwindow.
     */
    public enum View {
        tournamentList("tournamentList/tournamentList.fxml"),
        participantFormular("participantAddFormular/participantFormular.fxml"),
        placesFormular("placesAddFormular/PlacesFormular.fxml"),
        gameScheduler("gameScheduleView/GameSchedule.fxml");

        private String fxmlFileName;
        private Pane pane;

        /**
         * Constructor of View
         * @param fxmlFileName The name of the FXML File to load.
         */
        private View(String fxmlFileName) {
            this.fxmlFileName = fxmlFileName;
        }

        public Pane getPane() {
            return pane;
        }

        /**
         * Method to laod the view
         * @param tournamentDecorator the tournamentDecorator object which will be passed to the controller.
         * @param fileIO the fileIO object which will be passed to the controller.
         * @param factoryDecorator the factoryDecorator object which will be passed to the controller.
         * @param borderPane the borderPane object which will be passed to the controller.
         * @throws IOException if the fxml file is not found or can not be loaded correctly.
         */
        public void loadView(TournamentDecorator tournamentDecorator, FileIO fileIO, FactoryDecorator factoryDecorator, BorderPane borderPane) throws IOException {
            FXMLLoader loader = new FXMLLoader(getClass().getResource(fxmlFileName));
            this.pane = loader.load();
            FXController controller = loader.getController();
            controller.setup(tournamentDecorator, fileIO, factoryDecorator, borderPane, languageConfigurator);
        }

    }

}
