package ch.zhaw.projekt2.turnierverwaltung;


import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Path;
import java.util.Comparator;
import java.util.List;


import static org.junit.jupiter.api.Assertions.*;

/**
 * FileIO Test Class
 */
class FileIOTest {

    String RESOURCES_DIR = "./src/test/resources/ch/zhaw/projekt2/turnierverwaltung/";
    String mainDir;
    String saveDir;
    FileIO io;

    /**
     * Method checks if a new directory is set up correctly and can be accessed.
     *
     * @throws IOException Exceptions should not be thrown
     */
    @Test
    void FileIONewDir() throws IOException {
        mainDir = RESOURCES_DIR + "FileIONew";
        saveDir = mainDir + "/saves";
        File mainDirFile = new File(mainDir);
        File saveDirFile = new File(mainDir);
        try{
            Files.walk(mainDirFile.toPath())
                    .sorted(Comparator.reverseOrder())
                    .map(Path::toFile)
                    .forEach(File::delete);
        } catch (IOException e) {
            e.printStackTrace();
        }
        assertFalse(mainDirFile.exists());
        assertFalse(saveDirFile.exists());
        io = new FileIO(mainDir);
        assertTrue(mainDirFile.exists());
        assertTrue(saveDirFile.exists());
        Files.walk(mainDirFile.toPath())
                .sorted(Comparator.reverseOrder())
                .map(Path::toFile)
                .forEach(File::delete);
        assertFalse(mainDirFile.exists());
        assertFalse(saveDirFile.exists());
    }

    /**
     * Method tests the read behavior and if a file is read correctly
     */
    @Nested
    class Read{
        /**
         *  Sets up a directory
         */
        @BeforeEach
        void init() {
            mainDir = RESOURCES_DIR + "FileIORead";
            io = new FileIO(mainDir);
        }

        /**
         * Test if the list is displayed correctly when getting it via getList
         */
        @Test
        void getList() {
            List<FileIO.TournamentFile> tournaments = io.getList();
            assertEquals("empty.txt", tournaments.get(0).getName());
            assertEquals("test1.txt", tournaments.get(1).getName());
        }

        /**
         * Test behaviour when the list is empty
         */
        @Test
        void getListEmpty() {
            mainDir = RESOURCES_DIR + "FileIOEmpty";
            io = new FileIO(mainDir);
            assertEquals(0, io.getList().size());
        }

        /**
         * Tests behavior when loading a tournament that exists.
         *
         * @throws IOException Exceptions should not be thrown
         * @throws ClassNotFoundException Exceptions should not be thrown
         */
        @Test
        void loadTournament() throws IOException, ClassNotFoundException {
            mainDir = RESOURCES_DIR + "FileIORead";
            io = new FileIO(mainDir);
            Tournament tournament = io.loadTournament(new File(mainDir + "/saves/test1.txt"));
            assertEquals("test1", tournament.getName());
        }

        /**
         * Test behavior when trying to load non-existent tournament
         */
        @Test
        void loadTournamentNotExisting(){
            File file = new File("Not-existing-File");
            assertFalse(file.exists());
            assertThrows(FileNotFoundException.class, () -> io.loadTournament(file));
            assertFalse(file.exists());
        }

        /**
         * Tests behavior when trying to load an empty tournament.
         */
        @Test
        void loadTournamentEmpty(){
            assertThrows(IOException.class, () -> io.loadTournament(new File(mainDir + "/saves/empty.txt")));
        }

        /**
         * Tests behavior when the tournamentfile input is null
         */
        @Test
        void loadTournamentFileNull(){
            assertThrows(IllegalArgumentException.class, () -> io.loadTournament(null));
        }
    }

    @Nested
    class Save{
        @BeforeEach
        void setup(){
            mainDir = RESOURCES_DIR + "FileIOSave";
            io = new FileIO(mainDir);
        }

        /**
         * Saves the Saving mechanism and deletion
         *
         * @throws IOException Exceptions should not be thrown
         * @throws InvalidNameException Exceptions should not be thrown
         * @throws Tournament.InvalidTypeException Exceptions should not be thrown
         */
        @Test
        void saveTournament() throws IOException, InvalidNameException, Tournament.InvalidTypeException {
            Tournament tournament = null;
            tournament = new Tournament("test1", Tournament.Type.KO);
            io.saveTournament(tournament);
            File file = new File(mainDir + "/saves/test1.txt");
            assertTrue(file.exists());
            assertTrue(file.delete());
            assertFalse(file.exists());
        }

        /**
         * Tests behavior when a tournament is being saved that is only null
         */
        @Test
        void saveTournamentNull(){
            assertThrows(IllegalArgumentException.class, () -> io.saveTournament(null));
        }
    }

    @Nested
    class Delete{
        @BeforeEach
        void setup(){
            mainDir = RESOURCES_DIR + "FileIODelete";
            io = new FileIO(mainDir);
        }

        /**
         *  Test if tournament that does exist can be deleted
         * @throws IOException Exceptions should not be thrown
         */
        @Test
        void deleteTournament() throws IOException {
            File file = new File(mainDir + "/saves/test1.txt");
            file.createNewFile();
            assertTrue(file.exists());
            io.deleteTournament(file);
            assertFalse(file.exists());
        }

        /**
         * Testing if tournament that does not exist can be deleted
         *
         * @throws IOException Exception should not be thrown only checking for FileNotFoundException
         */
        @Test
        void deleteTournamentNotExisting() throws IOException {
            File file = new File("Not-existing-File");
            assertFalse(file.exists());
            assertThrows(FileNotFoundException.class, () -> io.deleteTournament(file));
            assertFalse(file.exists());
        }

        /**
         * Tests if a tournament that is null can be deleted
         */
        @Test
        void deleteTournamentNull(){
            assertThrows(IllegalArgumentException.class, () -> io.deleteTournament(null));
        }
    }
}